##    Chelydra. A zip-based backup and restore tool.
##    Copyright (C) 2023  Erick Fernando Mora Ramirez
##
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <https://www.gnu.org/licenses/>.
##
##    mailto:erickfernandomoraramirez@gmail.com

from dataclasses import replace
from zipfile import ZipFile

def update(compressed_file_name:str, entry_name:str, absolute_filename:str):
    with open(absolute_filename, 'rb') as fi:
        update_bytes(compressed_file_name, entry_name, fi.read())

def update_bytes(compressed_file_name:str, entry_name:str, data:bytes):
    zipfile = ZipFile(compressed_file_name, 'a')
    with zipfile.open(entry_name, 'w') as fo:
        fo.write(data)

def get_entries(compressed_file_name:str):
    zipfile = ZipFile(compressed_file_name, 'r')
    for zipinfo in zipfile.filelist:
        yield zipinfo.filename

def extract_to_folder(compressed_file_name:str, entry_name:str, folder_name:str):
    entry_name = entry_name.replace('\\', '/')
    ZipFile(compressed_file_name, 'r').extract(entry_name, folder_name)


##    Chelydra. A zip-based backup and restore tool.
##    Copyright (C) 2023  Erick Fernando Mora Ramirez
##
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <https://www.gnu.org/licenses/>.
##
##    mailto:erickfernandomoraramirez@gmail.com

from typing import Any, Dict, List

VERSION_MODE_FULL = 'FULL'
VERSION_MODE_PART = 'PART'

class VersionId:
    def __init__(self, mode:str = None, code:str = None, epoch:float = None, order:int = None) -> None:
        self.mode = mode or VERSION_MODE_FULL
        self.code = code or ''
        self.epoch = epoch or 0.0
        self.order = order or 0
        if self.mode not in [VERSION_MODE_FULL, VERSION_MODE_PART]:
            raise Exception('Invalid mode')

class Version:
    def __init__(self, id:VersionId = None, changes:Dict[str, str] = None, deletions:List[str] = None) -> None:
        self.id = id or VersionId()
        self.changes = changes or {}
        self.deletions = deletions or []

    def is_full(self) -> bool:
        return self.id.mode == VERSION_MODE_FULL

    def load(self, d:Dict[str, Any]):
        self.id.mode = d['mode']
        self.id.code = d['code']
        self.id.epoch = d['epoch']
        self.id.order = d['order']
        self.changes = d['changes']
        self.deletions = d['deletions']
        return self

    def get_dict(self)->Dict[str, Any]:
        return {
            "mode": self.id.mode,
            "code": self.id.code,
            "epoch": self.id.epoch,
            "order": self.id.order,
            "changes": self.changes,
            "deletions": self.deletions
        }

    def get_filename(self, posfix:str = None):
        posfix = posfix or ''
        order = str(self.id.order).rjust(8, '0')
        mode = self.id.mode
        code = self.id.code
        return f'[{mode}][{order}][{code}]{posfix}'


##    Chelydra. A zip-based backup and restore tool.
##    Copyright (C) 2023  Erick Fernando Mora Ramirez
##
##    This program is free software: you can redistribute it and/or modify
##    it under the terms of the GNU General Public License as published by
##    the Free Software Foundation, either version 3 of the License, or
##    (at your option) any later version.
##
##    This program is distributed in the hope that it will be useful,
##    but WITHOUT ANY WARRANTY; without even the implied warranty of
##    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
##    GNU General Public License for more details.
##
##    You should have received a copy of the GNU General Public License
##    along with this program.  If not, see <https://www.gnu.org/licenses/>.
##
##    mailto:erickfernandomoraramirez@gmail.com

from hashlib import md5
from typing import List

def get_file_hash(absolute_path:str) -> str:
    with open(absolute_path, 'rb') as f:
        hash_obj = md5()
        hash_obj.update(f.read())
        return hash_obj.hexdigest()

def get_list_hash(data:List[str], encoding:str = 'latin1') -> str:
    hash_obj = md5()
    for chunk in sorted(data):
        hash_obj.update(chunk.encode(encoding))
    return hash_obj.hexdigest()